package main

import androidx.compose.foundation.layout.*
import androidx.compose.material.*
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.unit.dp
import org.apache.poi.ss.usermodel.CellType
import org.apache.poi.ss.usermodel.Workbook
import org.apache.poi.xssf.usermodel.XSSFWorkbook
import java.awt.FileDialog
import java.awt.Frame
import java.io.FileInputStream
import java.io.FileWriter
import java.io.IOException
import java.io.File


data class Glossary(val name: String, val jsonFilePath: String)

val frame = Frame()
@OptIn(ExperimentalMaterialApi::class)
@Composable
fun glossaryPage(
    languageManager: LanguageManager,
    onAddWordClick: () -> Unit,
    onImportClick: () -> Unit,
    onExportClick: () -> Unit,
    onSeeGlossaryClick: () -> Unit,
    glossary: Glossary,
    onBackClick: () -> Unit,
) {


    Column(
        modifier = Modifier.fillMaxSize(),
        verticalArrangement = Arrangement.Top, // Align content at the top
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        Text(text = languageManager.getGlossaryTitle() + ": " + glossary.name, style = MaterialTheme.typography.h3)
    }

    Column(
        modifier = Modifier
            .fillMaxSize()  // Fills the maximum available width
            .padding(16.dp),
        verticalArrangement = Arrangement.Center,
        horizontalAlignment = Alignment.CenterHorizontally

    ) {

        Button(
            onClick = onAddWordClick,
            modifier = Modifier
                .width(300.dp),
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )
        ) {
            Text(languageManager.getAddWordText())
        }

        Button(
            onClick = onImportClick,
            modifier = Modifier
                .width(300.dp),
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )
        ) {
            Text(languageManager.getImportText())
        }

        if (importedSuccessfully.value) {
            AlertDialog(
                onDismissRequest = { importedSuccessfully.value = false },
                title = { Text("Importation réussie") },
                text = { Text("Le fichier a été importé avec succès") },
                confirmButton = {
                    Button(
                        onClick = {
                            importedSuccessfully.value = false
                            frame.dispose()
                        },
                        colors = ButtonDefaults.buttonColors(
                            backgroundColor = customRedColor,
                            contentColor = Color.White
                        )
                    ) {
                        Text("OK")
                    }
                }
            )

        }

            if (errorImportation.value) {
                AlertDialog(
                    onDismissRequest = { errorImportation.value = false },
                    title = { Text("Importation Echouée") },
                    text = { Text("Le fichier n'a pas pu être importé") },
                    confirmButton = {
                        Button(
                            onClick = {
                                errorImportation.value = false
                                frame.dispose()
                            },
                            colors = ButtonDefaults.buttonColors(
                                backgroundColor = customRedColor,
                                contentColor = Color.White
                            )
                        ) {
                            Text("OK")
                        }
                    }
                )
            }


            Button(
            onClick = onExportClick,
            modifier = Modifier
                .width(300.dp),
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )
        ) {
            Text(languageManager.getExportText())
        }

        if (exportedSuccessfully.value) {
            AlertDialog(
                onDismissRequest = { exportedSuccessfully.value = false },
                title = { Text("Exportation réussie") },
                text = { Text("Le fichier a été exporté avec succès") },
                confirmButton = {
                    Button(
                        onClick = { exportedSuccessfully.value = false },
                        colors = ButtonDefaults.buttonColors(
                            backgroundColor = customRedColor,
                            contentColor = Color.White
                        )
                    ) {
                        Text("OK")
                    }
                }
            )
        }

        Button(
            onClick = onSeeGlossaryClick,
            modifier = Modifier
                .width(300.dp),
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )
        ) {
            Text(languageManager.getSeeGlossaryText())
        }
    }

    Column(
        modifier = Modifier.fillMaxSize().padding(20.dp),
        verticalArrangement = Arrangement.Bottom,
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        Button(
            onClick = onBackClick,
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )
        ) {
            Text(languageManager.getBackButtonText())
        }
    }
}

var importedSuccessfully = mutableStateOf(false)
var errorImportation = mutableStateOf(false)
var exportedSuccessfully = mutableStateOf(false)
fun selectFile(extensions: Set<String>, onFileSelected: (String) -> Unit) {

    val fileDialog = FileDialog(frame, "Select a file", FileDialog.LOAD)
    fileDialog.isMultipleMode = true // To enable selecting only one file
    fileDialog.file = "*." + extensions.joinToString(";*.")
    fileDialog.isVisible = true

    val selectedFiles = fileDialog.files

    if (selectedFiles != null) {
        for (file in selectedFiles) {
            println("Selected file: $file")
            // Verify that the file extension is valid
            val fileExtension = File(file.absolutePath).extension.lowercase()
            if (extensions.contains(fileExtension)) {
                println("Opening: $file")
                onFileSelected(file.absolutePath)
            } else {
                println("Invalid file extension.")
            }
        }
    } else {
        println("Open command cancelled by user.")
    }
    frame.dispose()
}

fun exportToCSV(glossary: Glossary, csvFilePath: String) {
    val glossary = loadDatasFromFile(glossary.jsonFilePath)
    val csvContent = buildString {
        appendLine("Mot;Description;Contexte principal;Contexte 2;Lié à;Synonyme;Antonyme;")
        glossary.forEach { entry ->
            appendLine(
                "${entry.name};${entry.description};${entry.mainContext};" +
                        "${entry.secondaryContext};${entry.relatedTo};${entry.synonymous};${entry.antonym}"
            )
        }
    }
    try {
        FileWriter(csvFilePath + ".csv").use { fileWriter ->
            fileWriter.write(csvContent)
        }
        exportedSuccessfully.value = true
    } catch (e: IOException) {
        e.printStackTrace()
    }
}

fun importFile(glossary: Glossary, filePath: String) {
    val fileExtension = File(filePath).extension.lowercase()

    when (fileExtension) {
        "csv" -> {
            importCSVFile(glossary, filePath)
        }

        "xlsx" -> {
            importXLSXFile(glossary, filePath)
        }

        else -> {
            println("Unsupported file format.")
        }
    }
}


fun importCSVFile(glossary : Glossary, filePath: String) {
    val file = File(filePath).readText(Charsets.UTF_8)

    val lines = file.split("\n")
    val header = lines.first().split(";")
    val dataLines = lines.drop(1)

    dataLines.forEach { line ->
        // Check if the line is not empty or blank
        if (line.isNotBlank()) {
            val values = line.replace("\r", "").replace("\n", "").split(";")

            values.forEach { value ->
                println(value)
            }
            val mot = header.zip(values).toMap().toMutableMap()
            try {
                val nouveauWord = Word(
                    name = mot["Mot"]!!,
                    description = mot["Description"]!!,
                    mainContext = mot["Contexte principal"]!!,
                    secondaryContext = mot["Contexte 2"]!!,
                    relatedTo = mot["Lié à"]!!,
                    synonymous = mot["Synonyme"]!!,
                    antonym = mot["Antonyme"]!!
                )

                if (mot["Antonyme"] == "\r") {
                    mot["Antonyme"] = ""
                }
                if (mot["Mot"] == null || mot["Mot"] == "") {
                    return
                }
                if (!verifyRequiredFields(mot)) {
                    return
                }
                addWordToGlossary(glossary.jsonFilePath, nouveauWord)
            }catch(e : NullPointerException){
                errorImportation.value = true
            }

        }
        importedSuccessfully.value = true
    }

}


private fun importXLSXFile(glossary: Glossary, cheminFichier: String) {
    val workbook: Workbook
    try {
        FileInputStream(cheminFichier).use { fileInputStream ->
            workbook = XSSFWorkbook(fileInputStream)
        }

        val sheet = workbook.getSheetAt(0) // Assuming the data is in the first sheet

        val headerRow = sheet.getRow(0)
        val header = mutableListOf<String>()
        for (i in 0 until headerRow.lastCellNum) {
            header.add(headerRow.getCell(i).stringCellValue)
        }

        val dataLines = mutableListOf<Map<String, String>>()

        for (i in 1 until sheet.physicalNumberOfRows) {
            val currentRow = sheet.getRow(i)
            val data = mutableMapOf<String, String>()

            for (j in 0 until currentRow.lastCellNum) {
                val cell = currentRow.getCell(j)
                val cellValue = when (cell.cellType) {
                    CellType.STRING -> cell.stringCellValue
                    CellType.NUMERIC -> cell.numericCellValue.toString()
                    else -> ""
                }
                data[header[j]] = cellValue
            }

            dataLines.add(data)
        }

        dataLines.forEach { line ->
            // Check if the line is not empty or blank
            if (line.isNotEmpty()) {
                // Process each line as needed
                val newWord = Word(
                    name = line["Mot"] ?: "",
                    description = line["Description"] ?: "",
                    mainContext = line["Contexte principal"] ?: "",
                    secondaryContext = line["Contexte 2"] ?: "",
                    relatedTo = line["Lié à"] ?: "",
                    synonymous = line["Synonyme"] ?: "",
                    antonym = line["Antonyme"] ?: ""
                )

                if (newWord.name.isNotBlank() && verifyRequiredFields(line)) {
                    addWordToGlossary(glossary.jsonFilePath, newWord)
                }
            }
        }
    } catch (e: IOException) {
        e.printStackTrace()
    }
}

fun verifyRequiredFields(mot: Map<String, String>): Boolean {
    //Verify that the required fields are present
    val requiredFields = listOf(
        "Mot",
        "Description",
        "Contexte principal",
        "Contexte 2",
        "Lié à",
        "Synonyme",
        "Antonyme"
    )

    val motKeys = mot.keys.map { it.trim() }

    motKeys.forEach { key ->
        if (!requiredFields.contains(key)) {
            println("Le champ $key n'est pas reconnu")
            return false
        }
    }

    return true

}